*** a/src/Makefile.global.in
--- b/src/Makefile.global.in
***************
*** 517,522 **** $(top_builddir)/src/interfaces/ecpg/include/ecpg_config.h: $(top_builddir)/src/i
--- 517,592 ----
  $(top_builddir)/config.status: $(top_srcdir)/configure
  	cd $(top_builddir) && ./config.status --recheck
  
+ # Note: there are some undocumented dependencies on the ordering in which
+ # the catalog header files are assembled into postgres.bki.  In particular,
+ # indexing.h had better be last, and toasting.h just before it.
+ 
+ POSTGRES_BKI_SRCS = $(addprefix $(top_srcdir)/src/include/catalog/,\
+ 	pg_proc.h pg_type.h pg_attribute.h pg_class.h \
+ 	pg_attrdef.h pg_constraint.h pg_inherits.h pg_index.h pg_operator.h \
+ 	pg_opfamily.h pg_opclass.h pg_am.h pg_amop.h pg_amproc.h \
+ 	pg_language.h pg_largeobject.h pg_aggregate.h pg_statistic.h \
+ 	pg_rewrite.h pg_trigger.h pg_listener.h pg_description.h pg_cast.h \
+ 	pg_enum.h pg_namespace.h pg_conversion.h pg_depend.h \
+ 	pg_database.h pg_tablespace.h pg_pltemplate.h \
+ 	pg_authid.h pg_auth_members.h pg_shdepend.h pg_shdescription.h \
+ 	pg_ts_config.h pg_ts_config_map.h pg_ts_dict.h \
+ 	pg_ts_parser.h pg_ts_template.h \
+ 	pg_foreign_data_wrapper.h pg_foreign_server.h pg_user_mapping.h \
+ 	toasting.h indexing.h \
+     )
+ 
+ # Generated header files need dependencies here to ensure that everything
+ # which depends on them gets rebuilt when necessary.  Note that it's important
+ # we match the dependencies shown in the subdirectory makefiles!
+ ifneq ($(subdir),src/backend/parser)
+ $(top_srcdir)/src/backend/parser/gram.h: $(top_srcdir)/src/backend/parser/gram.y
+ 	$(MAKE) -C $(top_builddir)/src/backend/parser gram.h
+ endif
+ 
+ ifneq ($(subdir),src/backend/catalog)
+ # Same logic as pg_config.h and ecpg_config.h
+ $(top_srcdir)/src/backend/catalog/fmgroids.h: $(top_srcdir)/src/backend/catalog/stamp-h ;
+ 
+ $(top_srcdir)/src/backend/catalog/schemapg.h: $(top_srcdir)/src/backend/catalog/stamp-h;
+ 
+ $(top_srcdir)/src/backend/catalog/stamp-h: $(top_srcdir)/src/backend/catalog/gen_catalog.pl $(POSTGRES_BKI_SRCS)
+ 	$(MAKE) -C $(top_builddir)/src/backend/catalog stamp-h
+ endif
+ 
+ ifneq ($(subdir),src/backend/utils)
+ $(top_builddir)/src/backend/utils/probes.h: $(top_srcdir)/src/backend/utils/probes.d
+ 	$(MAKE) -C $(top_builddir)/src/backend/utils probes.h
+ endif
+ 
+ # Make symlinks for these headers in the include directory. That way
+ # we can cut down on the -I options. Also, a symlink is automatically
+ # up to date when we update the base file.
+ #
+ # The point of the prereqdir incantation in some of the rules below is to force
+ # the symlink to use an absolute path rather than a relative path.  For headers
+ # which are generated by make distprep, the copy within src/backend will be in
+ # the source tree, and the copy in src/include will be in the build tree.
+ 
+ $(top_builddir)/src/include/parser/gram.h: $(top_srcdir)/src/backend/parser/gram.h
+ 	prereqdir=`cd $(dir $<) >/dev/null && pwd` && \
+ 	  cd $(dir $@) && rm -f $(notdir $@) && \
+ 	  $(LN_S) "$$prereqdir/$(notdir $<)" .
+ 
+ $(top_builddir)/src/include/utils/fmgroids.h: $(top_srcdir)/src/backend/catalog/fmgroids.h
+ 	prereqdir=`cd $(dir $<) >/dev/null && pwd` && \
+ 	  cd $(dir $@) && rm -f $(notdir $@) && \
+ 	  $(LN_S) "$$prereqdir/$(notdir $<)" .
+ 
+ $(top_builddir)/src/include/catalog/schemapg.h: $(top_srcdir)/src/backend/catalog/schemapg.h
+ 	prereqdir=`cd $(dir $<) >/dev/null && pwd` && \
+ 	  cd $(dir $@) && rm -f $(notdir $@) && \
+ 	  $(LN_S) "$$prereqdir/$(notdir $<)" .
+ 
+ $(top_builddir)/src/include/utils/probes.h: $(top_builddir)/src/backend/utils/probes.h
+ 	cd $(dir $@) && rm -f $(notdir $@) && \
+ 	    $(LN_S) ../../../src/backend/utils/probes.h .
+ 
  endif # not PGXS
  
  
*** a/src/backend/Makefile
--- b/src/backend/Makefile
***************
*** 105,150 **** endif
  endif # aix
  
  # Update the commonly used headers before building the subdirectories
! $(SUBDIRS:%=%-recursive): $(top_builddir)/src/include/parser/gram.h $(top_builddir)/src/include/utils/fmgroids.h $(top_builddir)/src/include/utils/probes.h
! 
  
  # The postgres.o target is needed by the rule in Makefile.global that
  # creates the exports file when MAKE_EXPORTS = true.
  postgres.o: $(OBJS)
  	$(CC) $(LDREL) $(LDFLAGS) $(call expand_subsys,$^) $(LIBS) -o $@
  
- 
- # The following targets are specified in make commands that appear in
- # the make files in our subdirectories. Note that it's important we
- # match the dependencies shown in the subdirectory makefiles!
- 
- $(srcdir)/parser/gram.h: parser/gram.y
- 	$(MAKE) -C parser gram.h
- 
- utils/fmgroids.h: utils/Gen_fmgrtab.sh $(top_srcdir)/src/include/catalog/pg_proc.h
- 	$(MAKE) -C utils fmgroids.h
- 
- utils/probes.h: utils/probes.d
- 	$(MAKE) -C utils probes.h
- 
- # Make symlinks for these headers in the include directory. That way
- # we can cut down on the -I options. Also, a symlink is automatically
- # up to date when we update the base file.
- 
- $(top_builddir)/src/include/parser/gram.h: $(srcdir)/parser/gram.h
- 	prereqdir=`cd $(dir $<) >/dev/null && pwd` && \
- 	  cd $(dir $@) && rm -f $(notdir $@) && \
- 	  $(LN_S) "$$prereqdir/$(notdir $<)" .
- 
- $(top_builddir)/src/include/utils/fmgroids.h: utils/fmgroids.h
- 	cd $(dir $@) && rm -f $(notdir $@) && \
- 	    $(LN_S) ../../../$(subdir)/utils/fmgroids.h .
- 
- $(top_builddir)/src/include/utils/probes.h: utils/probes.h
- 	cd $(dir $@) && rm -f $(notdir $@) && \
- 	    $(LN_S) ../../../$(subdir)/utils/probes.h .
- 
- 
  ifeq ($(PORTNAME), solaris)
  utils/probes.o: utils/probes.d $(SUBDIROBJS)
  	$(DTRACE) $(DTRACEFLAGS) -C -G -s $(call expand_subsys,$^) -o $@
--- 105,117 ----
  endif # aix
  
  # Update the commonly used headers before building the subdirectories
! $(SUBDIRS:%=%-recursive): $(top_builddir)/src/include/parser/gram.h $(top_builddir)/src/include/utils/fmgroids.h $(top_builddir)/src/include/utils/probes.h $(top_builddir)/src/include/catalog/schemapg.h
  
  # The postgres.o target is needed by the rule in Makefile.global that
  # creates the exports file when MAKE_EXPORTS = true.
  postgres.o: $(OBJS)
  	$(CC) $(LDREL) $(LDFLAGS) $(call expand_subsys,$^) $(LIBS) -o $@
  
  ifeq ($(PORTNAME), solaris)
  utils/probes.o: utils/probes.d $(SUBDIROBJS)
  	$(DTRACE) $(DTRACEFLAGS) -C -G -s $(call expand_subsys,$^) -o $@
***************
*** 157,162 **** distprep:
--- 124,131 ----
  	$(MAKE) -C parser	gram.c gram.h scan.c
  	$(MAKE) -C bootstrap	bootparse.c bootscanner.c
  	$(MAKE) -C utils/misc	guc-file.c
+ 	$(MAKE) -C catalog	postgres.bki postgres.description \
+ 		postgres.shdescription fmgroids.h fmgrtab.c schemapg.h
  
  
  ##########################################################################
*** /dev/null
--- b/src/backend/catalog/.cvsignore
***************
*** 0 ****
--- 1,2 ----
+ fmgrtab.c
+ fmgroids.h
*** a/src/backend/catalog/Makefile
--- b/src/backend/catalog/Makefile
***************
*** 10,54 **** subdir = src/backend/catalog
  top_builddir = ../../..
  include $(top_builddir)/src/Makefile.global
  
! OBJS = catalog.o dependency.o heap.o index.o indexing.o namespace.o aclchk.o \
!        pg_aggregate.o pg_constraint.o pg_conversion.o pg_depend.o pg_enum.o \
!        pg_inherits.o pg_largeobject.o pg_namespace.o pg_operator.o pg_proc.o \
!        pg_shdepend.o pg_type.o storage.o toasting.o
  
  BKIFILES = postgres.bki postgres.description postgres.shdescription
  
  include $(top_srcdir)/src/backend/common.mk
  
! all: $(BKIFILES)
! 
! # Note: there are some undocumented dependencies on the ordering in which
! # the catalog header files are assembled into postgres.bki.  In particular,
! # indexing.h had better be last, and toasting.h just before it.
! 
! POSTGRES_BKI_SRCS = $(addprefix $(top_srcdir)/src/include/catalog/,\
! 	pg_proc.h pg_type.h pg_attribute.h pg_class.h \
! 	pg_attrdef.h pg_constraint.h pg_inherits.h pg_index.h pg_operator.h \
! 	pg_opfamily.h pg_opclass.h pg_am.h pg_amop.h pg_amproc.h \
! 	pg_language.h pg_largeobject.h pg_aggregate.h pg_statistic.h \
! 	pg_rewrite.h pg_trigger.h pg_listener.h pg_description.h pg_cast.h \
! 	pg_enum.h pg_namespace.h pg_conversion.h pg_depend.h \
! 	pg_database.h pg_tablespace.h pg_pltemplate.h \
! 	pg_authid.h pg_auth_members.h pg_shdepend.h pg_shdescription.h \
! 	pg_ts_config.h pg_ts_config_map.h pg_ts_dict.h \
! 	pg_ts_parser.h pg_ts_template.h \
! 	pg_foreign_data_wrapper.h pg_foreign_server.h pg_user_mapping.h \
! 	toasting.h indexing.h \
!     )
  
  pg_includes = $(sort -I$(top_srcdir)/src/include -I$(top_builddir)/src/include)
  
! # see explanation in ../parser/Makefile
! postgres.description: postgres.bki ;
  
! postgres.shdescription: postgres.bki ;
  
! postgres.bki: genbki.sh $(POSTGRES_BKI_SRCS) $(top_builddir)/src/include/pg_config_manual.h
! 	AWK='$(AWK)' $(SHELL) $< $(pg_includes) --set-version=$(VERSION) -o postgres $(POSTGRES_BKI_SRCS)
  
  .PHONY: install-data
  install-data: $(BKIFILES) installdirs
--- 10,45 ----
  top_builddir = ../../..
  include $(top_builddir)/src/Makefile.global
  
! OBJS = catalog.o dependency.o fmgrtab.o heap.o index.o indexing.o namespace.o \
! 	   aclchk.o pg_aggregate.o pg_constraint.o pg_conversion.o pg_depend.o \
! 	   pg_enum.o pg_inherits.o pg_largeobject.o pg_namespace.o pg_operator.o \
! 	   pg_proc.o pg_shdepend.o pg_type.o storage.o toasting.o
  
  BKIFILES = postgres.bki postgres.description postgres.shdescription
  
  include $(top_srcdir)/src/backend/common.mk
  
! all: $(addprefix $(srcdir)/, $(BKIFILES) fmgroids.h fmgrtab.c schemapg.h)
  
  pg_includes = $(sort -I$(top_srcdir)/src/include -I$(top_builddir)/src/include)
  
! # see src/backend/parser/Makefile for notes on dependency chaining for multiple
! # output files; see src/Makefile.global.in for notes on how stamp-h files are
! # used
! $(srcdir)/postgres.bki: $(srcdir)/stamp-h ;
  
! $(srcdir)/postgres.description: $(srcdir)/stamp-h ;
  
! $(srcdir)/postgres.shdescription: $(srcdir)/stamp-h ;
! 
! $(srcdir)/fmgroids.h: $(srcdir)/stamp-h ;
! 
! $(srcdir)/fmgrtab.c: $(srcdir)/stamp-h ;
! 
! $(srcdir)/schemapg.h: $(srcdir)/stamp-h ;
! 
! $(srcdir)/stamp-h: gen_catalog.pl $(POSTGRES_BKI_SRCS) $(top_builddir)/src/include/pg_config_manual.h
! 	$(PERL) $< $(pg_includes) -o $(srcdir) --set-version=$(VERSION) $(POSTGRES_BKI_SRCS)
  
  .PHONY: install-data
  install-data: $(BKIFILES) installdirs
***************
*** 66,70 **** installdirs:
  uninstall-data:
  	rm -f $(addprefix '$(DESTDIR)$(datadir)'/, $(BKIFILES) system_views.sql information_schema.sql sql_features.txt)
  
! clean:
! 	rm -f $(BKIFILES)
--- 57,62 ----
  uninstall-data:
  	rm -f $(addprefix '$(DESTDIR)$(datadir)'/, $(BKIFILES) system_views.sql information_schema.sql sql_features.txt)
  
! # postgres.bki, postgres.description, postgres.shdescription,
! # fmgroids.h, fmgrtab.c, and schemapg.h are in the distribution tarball, so
! # they are not cleaned here.
*** /dev/null
--- b/src/backend/catalog/gen_catalog.pl
***************
*** 0 ****
--- 1,561 ----
+ #!/usr/bin/perl
+ 
+ #
+ # gen_catalog.pl
+ #
+ # Portions Copyright (c) 1996-2009, PostgreSQL Global Development Group
+ # Portions Copyright (c) 1994, Regents of the University of California
+ #
+ 
+ use strict;
+ use warnings;
+ 
+ #
+ # The purpose of this script is to generate as many of the files derived from
+ # the catalogs as possible in one script.  Since this script is written in
+ # Perl, it can be used under either Windows or UNIX-ish systems.  On the flip
+ # side, because none of what we generate here depends on architecture or
+ # configuration, we can build it all at distprep time so that, on UNIX, it
+ # is possible to build from an unmodified distribution tarball without Perl.
+ #
+ # However, if any of the relevant headers are modified, or when building from
+ # CVS (or git), Perl will be required, as this script must be rerun.
+ #
+ my @OUTPUT = (
+ 	[ 'postgres.bki', \*BKI ],
+ 	[ 'postgres.description', \*DESCR ],
+ 	[ 'postgres.shdescription', \*SHDESCR ],
+ 	[ 'fmgroids.h', \*FMGROIDS ],
+ 	[ 'fmgrtab.c', \*FMGRTAB ],
+ 	[ 'schemapg.h', \*SCHEMAPG ],
+ );
+ 
+ #
+ # There are a few types which are given one name in the C source, but a
+ # different name at the SQL level.  These are enumerated here.
+ #
+ my %RENAME_ATTTYPE = (
+ 	'Oid' => 'oid',
+ 	'NameData' => 'name',
+ 	'TransactionId' => 'xid'
+ );
+ 
+ #
+ # Argument parsing.  We could use GetOpt::Long or similar here, but doing it
+ # this way avoids depending on any outside modules.
+ #
+ my @include_path;
+ my @input_file;
+ my $output_path = '';
+ my $major_version;
+ while (@ARGV) {
+ 	my $arg = shift @ARGV;
+ 	if ($arg !~ /^-/) {
+ 		push @input_file, $arg;
+ 	}
+ 	elsif ($arg =~ /^-o/) {
+ 		$output_path = length($arg) > 2 ? substr($arg, 2) : shift @ARGV;
+ 	}
+ 	elsif ($arg =~ /^-I/) {
+ 		push @include_path, length($arg) > 2 ? substr($arg, 2) : shift @ARGV;
+ 	}
+ 	elsif ($arg =~ /^--set-version=(.*)$/) {
+ 		$major_version = $1;
+ 		# Remove minor version information, if any.
+ 		if ($major_version =~ /^(\d+.\d+)/) {
+ 			$major_version = $1;
+ 		}
+ 	}
+ 	else {
+ 		usage();
+ 	}
+ }
+ 
+ # Sanity check arguments.
+ die "No input files.\n" if ! @input_file;
+ die "No include path; you must specify -I at least once.\n" if ! @include_path;
+ die "No version specified.\n" if !defined $major_version;
+ 
+ #
+ # CAUTION: be wary about what symbols you substitute into the .bki file here!
+ # It's okay to substitute things that are expected to be really constant
+ # within a given Postgres release, such as fixed OIDs.  Do not substitute
+ # anything that could depend on platform or configuration.  (The right place
+ # to handle those sorts of things is in initdb.c's bootstrap_template1().)
+ #
+ my $BOOTSTRAP_SUPERUSERID =
+ 	find_defined_symbol('pg_authid.h', 'BOOTSTRAP_SUPERUSERID');
+ my $PG_CATALOG_NAMESPACE =
+ 	find_defined_symbol('pg_namespace.h', 'PG_CATALOG_NAMESPACE');
+ my $INTERNAL_LANGUAGE_ID =
+ 	find_defined_symbol('pg_language.h', 'INTERNALlanguageId');
+ 
+ # Make sure output_path ends in a slash.
+ if ($output_path ne '' && substr($output_path, -1) ne '/') {
+ 	$output_path .= '/';
+ }
+ 
+ # Open temporary output files.
+ for my $output (@OUTPUT) {
+ 	my $temp_name = $output_path . $output->[0] . '.tmp';
+ 	# We avoid using the three-argument form of open() here, because it is
+ 	# only supported in Perl 5.6 and higher.
+ 	open($output->[1], ">$temp_name") || die "$temp_name: $!";
+ }
+ 
+ # Opening boilerplate for postgres.bki
+ print BKI "# PostgreSQL $major_version\n";
+ 
+ # Opening boilerplate for fmgroids.h
+ print FMGROIDS <<EOM;
+ /*-------------------------------------------------------------------------
+  *
+  * fmgroids.h
+  *    Macros that define the OIDs of built-in functions.
+  *
+  * These macros can be used to avoid a catalog lookup when a specific
+  * fmgr-callable function needs to be referenced.
+  *
+  * Portions Copyright (c) 1996-2009, PostgreSQL Global Development Group
+  * Portions Copyright (c) 1994, Regents of the University of California
+  *
+  * NOTES
+  *	******************************
+  *	*** DO NOT EDIT THIS FILE! ***
+  *	******************************
+  *
+  *	It has been GENERATED by $0
+  *	from pg_proc.h
+  *
+  *-------------------------------------------------------------------------
+  */
+ #ifndef FMGROIDS_H
+ #define FMGROIDS_H
+ 
+ /*
+  *	Constant macros for the OIDs of entries in pg_proc.
+  *
+  *	NOTE: macros are named after the prosrc value, ie the actual C name
+  *	of the implementing function, not the proname which may be overloaded.
+  *	For example, we want to be able to assign different macro names to both
+  *	char_text() and name_text() even though these both appear with proname
+  *	'text'.  If the same C function appears in more than one pg_proc entry,
+  *	its equivalent macro will be defined with the lowest OID among those
+  *	entries.
+  */
+ EOM
+ 
+ # Opening boilerplate for fmgrtab.c
+ print FMGRTAB <<EOM;
+ /*-------------------------------------------------------------------------
+  *
+  * fmgrtab.c
+  *    The function manager's table of internal functions.
+  *
+  * Portions Copyright (c) 1996-2009, PostgreSQL Global Development Group
+  * Portions Copyright (c) 1994, Regents of the University of California
+  *
+  * NOTES
+  *
+  *	******************************
+  *	*** DO NOT EDIT THIS FILE! ***
+  *	******************************
+  *
+  *	It has been GENERATED by $0
+  *	from pg_proc.h
+  *
+  *-------------------------------------------------------------------------
+  */
+ 
+ #include "postgres.h"
+ 
+ #include "utils/fmgrtab.h"
+ 
+ EOM
+ 
+ # Opening boilerplate for schemapg.h
+ print SCHEMAPG <<EOM;
+ /*-------------------------------------------------------------------------
+  *
+  * schemapg.h
+  *    Schema_pg_xxx macros for use by relcache.c
+  *
+  * Portions Copyright (c) 1996-2009, PostgreSQL Global Development Group
+  * Portions Copyright (c) 1994, Regents of the University of California
+  *
+  * NOTES
+  *	******************************
+  *	*** DO NOT EDIT THIS FILE! ***
+  *	******************************
+  *
+  *	It has been GENERATED by $0
+  *
+  *-------------------------------------------------------------------------
+  */
+ #ifndef SCHEMAPG_H
+ #define SCHEMAPG_H
+ 
+ EOM
+ 
+ # Main part of the work: read and process each header file.
+ my (@fmgr, %schema_pg, %catalog_name);
+ for my $input_file (@input_file)
+ {
+ 	process_input_file($input_file);
+ }
+ 
+ # We're going to iterate over this array twice and it needs to be in sorted
+ # order both times.
+ @fmgr = sort { $a->{'oid'} <=> $b->{'oid'} } @fmgr;
+ 
+ # Emit fmgr #define's and extern's, but only one per prosrc value.
+ my %fmgr_seenit;
+ foreach my $s (@fmgr) {
+ 	next if $fmgr_seenit{$s->{'prosrc'}};
+ 	$fmgr_seenit{$s->{'prosrc'}} = 1;
+ 	print FMGROIDS "#define F_" . uc($s->{'prosrc'}) . " $s->{oid}\n";
+ 	print FMGRTAB "extern Datum " . $s->{'prosrc'} . " (PG_FUNCTION_ARGS);\n";
+ }
+ 
+ # Emit fmgr_builtins table.
+ print FMGRTAB "\nconst FmgrBuiltin fmgr_builtins[] = {\n";
+ my %bmap;
+ $bmap{'t'} = 'true';
+ $bmap{'f'} = 'false';
+ foreach my $s (@fmgr) {
+     print FMGRTAB join('',
+ 		'  { ', $s->{'oid'}, ', "', $s->{'prosrc'}, '", ', $s->{'nargs'},
+ 		', ', $bmap{$s->{'strict'}}, ', ', $bmap{$s->{'retset'}}, ', ',
+ 		$s->{'prosrc'}, " },\n");
+ }
+ 
+ # Emit Schema_pg_xxx declarations.
+ while (my ($oid, $entry_list) = each %schema_pg) {
+ 	my $catalog = $catalog_name{$oid};
+ 	die "catalog with oid $oid not found" if !defined $catalog;
+ 	print SCHEMAPG "#define Schema_$catalog \\\n",
+ 		join(", \\\n", @$entry_list), "\n\n";
+ }
+ 
+ # Closing boilerplate for fmgroids.h
+ print FMGROIDS "\n#endif /* FMGROIDS_H */\n";
+ 
+ # Closing boilerplate for fmgrtab.c
+ print FMGRTAB <<EOM;
+   /* dummy entry is easier than getting rid of comma after last real one */
+   /* (not that there has ever been anything wrong with *having* a
+      comma after the last field in an array initializer) */
+   { 0, NULL, 0, false, false, NULL }
+ };
+ 
+ /* Note fmgr_nbuiltins excludes the dummy entry */
+ const int fmgr_nbuiltins = (sizeof(fmgr_builtins) / sizeof(FmgrBuiltin)) - 1;
+ EOM
+ 
+ # Closing boilerplate for schemapg.h
+ print SCHEMAPG "#endif /* SCHEMAPG_H */\n";
+ 
+ # Close output files.
+ for my $output (@OUTPUT) {
+ 	my $temp_name = $output_path . $output->[0] . '.tmp';
+ 	close($output->[1]) || die "close: $temp_name: $!";
+ }
+ 
+ # Rename temporary files to final names, if anything has changed.
+ for my $output (@OUTPUT) {
+ 	my $temp_name = $output_path . $output->[0] . '.tmp';
+ 	my $final_name = $output_path . $output->[0];
+ 	if (-e $final_name && -s $temp_name == -s $final_name) {
+ 		open(TN, "<$temp_name") || die "$temp_name: $!";
+ 		if (open(FN, "<$temp_name")) {
+ 			local $/ = undef;
+ 			my $tn = <TN>;
+ 			my $fn = <FN>;
+ 			close(FN);
+ 			if ($tn eq $fn) {
+ 				print "$output->[0] unchanged, not replacing\n";
+ 				close(TN);
+ 				unlink($temp_name) || die "unlink: $temp_name: $!";
+ 				next;
+ 			}
+ 		}
+ 		close(TN);
+ 	}
+ 	rename($temp_name, $final_name) || die "rename: $temp_name: $!";
+ }
+ 
+ # Update timestamp file.  Makefile system uses this to avoid unnecessary
+ # rebuilds.
+ open(STAMP, ">$output_path/stamp-h") || die "stamp-h: $!";
+ close(STAMP);
+ 
+ exit 0;
+ 
+ #
+ # Read and process a single input file.
+ #
+ sub process_input_file {
+ 	my ($input_file) = @_;
+ 	open(INPUT_FILE, "<$input_file") || die "$input_file: $!";
+ 
+ 	# State we need to keep track of while scanning the file.
+ 	my ($oid, $catalog, $is_bootstrap, $declaring_attributes, $reln_open);
+ 	my (@bki_attr, @attname, %attname_to_attnum, %attname_to_atttype);
+ 
+ 	# Scan the input file.
+ 	while (<INPUT_FILE>) {
+ 		# Strip C-style comments.
+ 		s;/\*(.|\n)*\*/;;g;
+ 		if (m;/\*;) {
+ 			my $next_line = <INPUT_FILE>;
+ 			die "$input_file: ends within C-style comment\n"
+ 				if !defined $next_line;
+ 			$_ .= $next_line;
+ 			redo;
+ 		}
+ 
+ 		# Strip useless whitespace and trailing semicolons.
+ 		chomp;
+ 		s/^\s+//;
+ 		s/;\s*$//;
+ 		s/\s+/ /g;
+ 
+ 		#
+ 		# DATA lines are passed through to postgres.bki after stripping off
+ 		# the DATA( and the ) on the end.  Remember the OID for use by
+ 		# DESCR() and SHDESCR().
+ 		#
+ 		# In addition, when processing pg_proc.h, we extract all of the
+ 		# "internal" functions and save them so that we can eventually write
+ 		# them out to fmgroids.h and fmgrtab.h.
+ 		#
+ 		if (/^DATA\((insert(\s+OID\s+=\s+(\d+))?\s+\((.*)\))\s*\)$/) {
+ 			my ($data, $field_data) = ($1, $4);
+ 			# Save the OID, if any, for reference by a subsequent DESCR() or
+ 			# SHDESCR() declaration.
+ 			$oid = $3;
+ 			# Pass the (almost) raw data through to postgres.bki.
+ 			$data =~ s/\bPGUID\b/$BOOTSTRAP_SUPERUSERID/g;
+ 			$data =~ s/\bPGNSP\b/$PG_CATALOG_NAMESPACE/g;
+ 			print BKI $data, "\n";
+ 			# To construct fmgroids.h and fmgrtab.c, we need to inspect some
+ 			# of the individual data fields.  Just splitting on whitespace
+ 			# won't work, because some quoted fields might contain internal
+ 			# whitespace.  We handle this by folding them all to a simple
+ 			# "xxx". Fortunately, this script doesn't need to look at any
+ 			# fields that might need quoting, so this simple hack is
+ 			# sufficient.
+ 			if (defined $catalog) {
+ 				$field_data =~ s/^\s+//;
+ 				$field_data =~ s/"[^"]*"/"xxx"/g;
+ 				my @p = split /\s+/, $field_data;
+ 				# Emit fmgr declarations for non-internal functions.
+ 				if ($catalog eq 'pg_proc'
+ 					&& $p[$attname_to_attnum{'prolang'}-1] eq '12') {
+ 				    push @fmgr, {
+ 				        'oid'     => $oid,
+ 				        'proname' => $p[$attname_to_attnum{'proname'}-1],
+ 				        'strict'  => $p[$attname_to_attnum{'proisstrict'}-1],
+ 				        'retset'  => $p[$attname_to_attnum{'proretset'}-1],
+ 				        'nargs'   => $p[$attname_to_attnum{'pronargs'}-1],
+ 				        'prosrc'  => $p[$attname_to_attnum{'prosrc'}-1],
+ 					};
+ 				}
+ 				# Emit Schema_pg info for non-system attributes.
+ 				elsif ($catalog eq 'pg_attribute'
+ 					&& $p[$attname_to_attnum{'attnum'}-1] > 0) {
+ 					if (@attname != @p) {
+ 						die sprintf "catalog has %d attributes, DATA() has "
+ 							. "%d entries\n", 0+@attname, 0+@p;
+ 					}
+ 					my @cvalue;
+ 					for (my $i = 0; $i < @p; ++$i) {
+ 						my $attname = $attname[$i];
+ 						my $atttype = $attname_to_atttype{$attname};
+ 						if (grep { $atttype eq $_ } qw(oid int2 int4 float4)) {
+ 							push @cvalue, $p[$i];
+ 						}
+ 						elsif ($atttype eq 'aclitem[]') {
+ 							if ($p[$i] ne '_null_') {
+ 								die "can't handle non-null aclitem[]: $p[$i]";
+ 							}
+ 							push @cvalue, '{ 0 }';
+ 						}
+ 						elsif ($atttype eq 'bool') {
+ 							if ($p[$i] eq 't') {
+ 								push @cvalue, 'true';
+ 							}
+ 							elsif ($p[$i] eq 'f') {
+ 								push @cvalue, 'false';
+ 							}
+ 							else {
+ 								# May be a constant, like FLOAT4PASSBYVAL.
+ 								push @cvalue, $p[$i];
+ 							}
+ 						}
+ 						elsif ($atttype eq 'char') {
+ 							push @cvalue, "'$p[$i]'";
+ 						}
+ 						elsif ($atttype eq 'name') {
+ 							push @cvalue, "{\"$p[$i]\"}";
+ 						}
+ 						else {
+ 							die "unhandled type $atttype";
+ 						}
+ 					}
+ 				    my $attrelid = $p[$attname_to_attnum{'attrelid'}-1];
+ 					push @{$schema_pg{$attrelid}},
+ 						"{ " . join(", ", @cvalue) . " }";
+ 				}
+ 			}
+ 		}
+ 		# DESCR() and SHDESCR() lines apply to the most recent catalog
+ 		# and OID.
+ 		elsif (/^DESCR\(\"(.*)\"\)$/) {
+ 			my $data = $1;
+ 			if (!defined $catalog) {
+ 				die "DESCR() does not apply to any catalog ($input_file)";
+ 			}
+ 			elsif (!defined $oid) {
+ 				die "DESCR() does not apply to any oid ($input_file)";
+ 			}
+ 			elsif ($data ne '') {
+ 				printf DESCR "%d\t%s\t0\t%s\n", $oid, $catalog, $data;
+ 			}
+ 		}
+ 		elsif (/^SHDESCR\(\"(.*)\"\)$/) {
+ 			my $data = $1;
+ 			if (!defined $catalog) {
+ 				die "SHDESCR() does not apply to any catalog ($input_file)";
+ 			}
+ 			elsif (!defined $oid) {
+ 				die "SHDESCR() does not apply to any oid ($input_file)";
+ 			}
+ 			elsif ($data ne '') {
+ 				printf SHDESCR "%d\t%s\t%s\n", $oid, $catalog, $data;
+ 			}
+ 		}
+ 		elsif (/^DECLARE_(UNIQUE_)?INDEX\((\s*[\w+]*),\s*(\d+),\s*(.*)\)/) {
+ 			my ($is_unique, $iname, $oid, $data) = ($1, $2, $3, $4);
+ 			# close catalog before declaring index
+ 			if ($reln_open) {
+ 				$reln_open = 0;
+ 				print BKI "close $catalog\n";
+ 			}
+ 			printf BKI "declare %sindex %s %s %s\n",
+ 				$is_unique ? "unique " : "", $iname, $oid, $data;
+ 		}
+ 		elsif (/^DECLARE_TOAST\((\s*[\w+]*),\s*(\d+),\s*(\d+)\)/) {
+ 			my ($toastname, $toastoid, $indexoid) = ($1, $2, $3);
+ 			# close catalog before declaring toast
+ 			if ($reln_open) {
+ 				$reln_open = 0;
+ 				print BKI "close $catalog\n";
+ 			}
+ 			print BKI "declare toast $toastoid $indexoid on $toastname\n";
+ 		}
+ 		elsif (/^BUILD_INDICES/) {
+ 			print BKI "build indices\n";
+ 		}
+ 		elsif (/^CATALOG\(([^,]*),(.*)\)/) {
+ 			my ($catname, $catoid) = ($1, $2);
+ 			# close catalog before declaring next catalog
+ 			if ($reln_open) {
+ 				$reln_open = 0;
+ 				print BKI "close $catalog\n";
+ 			}
+ 			$catalog = $catname;
+ 			$is_bootstrap = /BKI_BOOTSTRAP/;
+ 			print BKI "create " . ($is_bootstrap ? 'bootstrap ' : '')
+ 				. (/BKI_SHARED_RELATION/ ? 'shared_relation ' : '')
+ 				. (/BKI_WITHOUT_OIDS/ ? 'without_oids ' : '')
+ 				. $catalog . ' ' . $catoid . "\n";
+ 			$declaring_attributes = 1;
+ 			undef %attname_to_attnum;
+ 			undef %attname_to_atttype;
+ 			undef @attname;
+ 			$catalog_name{$catoid} = $catname;
+ 		}
+ 		elsif ($declaring_attributes) {
+ 			if ($_ =~ /^{|^$/) {
+ 				;							# just ignore it
+ 			}
+ 			elsif ($_ =~ /^}/)	{
+ 				print BKI " (\n", join(" ,\n", @bki_attr), "\n )\n";
+ 				if (! $is_bootstrap) {
+ 					print BKI "open $catalog\n";
+ 				}
+ 				undef $declaring_attributes;
+ 				$reln_open = 1;
+ 			}
+ 			else {
+ 				my @datafields = split /\s+/, $_;
+ 				my $atttype = $datafields[0];
+ 				my $attname;
+ 				if (exists $RENAME_ATTTYPE{$atttype}) {
+ 					$atttype = $RENAME_ATTTYPE{$atttype};
+ 				}
+ 		        if ($datafields[1] =~ /(.*)\[.*\]/) {   # array attribute
+ 	                $attname = $1;
+ 	                $atttype .= "[]";  					# variable-length only
+ 				}
+ 				else {
+ 					$attname = $datafields[1];
+ 				}
+ 				push @bki_attr, " $attname = $atttype";
+ 				push @attname, $attname;
+ 				$attname_to_attnum{$attname} = 0+@bki_attr;
+ 				$attname_to_atttype{$attname} = $atttype;
+ 			}
+ 		}
+ 		elsif (/^(DATA|CATALOG|DECLARE_(INDEX|UNIQUE_INDEX|TOAST))/) {
+ 			die "malformed $1 line ($input_file)";
+ 		}
+ 	}
+ 	if ($reln_open) {
+ 		print BKI "close $catalog\n";
+ 	}
+ 	close(INPUT_FILE);
+ }
+ 
+ #
+ # Find a symbol defined in a particular header file and extract the value.
+ #
+ sub find_defined_symbol {
+ 	my ($catalog_header, $symbol) = @_;
+ 	for my $path (@include_path) {
+ 		my $file = $path . '/catalog/' . $catalog_header;
+ 		next if ! -f $file;
+ 		# We avoid using the three-argument form of open() here, because it is
+ 		# only supported in Perl 5.6 and higher.
+ 		open(FIND_DEFINED_SYMBOL, "<$file") || die "$file: $!";
+ 		while (<FIND_DEFINED_SYMBOL>) {
+ 			if (/^#define\s+\Q$symbol\E\s+(\S+)/) {
+ 				return $1;
+ 			}
+ 		}
+ 		close(FIND_DEFINED_SYMBOL);
+ 		die "$file: no definition found for $symbol\n";
+ 	}
+ 	die "$catalog_header: not found in any include directory\n";
+ }
+ 
+ #
+ # Display usage message and exit.
+ #
+ sub usage
+ {
+ 	die <<EOM;
+ Usage: genbki.pl [options] header...
+ 
+ Options:
+ 	-I				path to include files
+ 	--bki			prefix for BKI output files
+ 	--set-version	PostgreSQL version number for initdb cross-check
+ 
+ genbki.pl generates .bki files from specially formatted .h files.  These .bki
+ files are used to initialize the postgres template database.
+ 
+ Report bugs to <pgsql-bugs\@postgresql.org>.
+ EOM
+ }
*** a/src/backend/catalog/genbki.sh
--- /dev/null
***************
*** 1,430 ****
- #! /bin/sh
- #-------------------------------------------------------------------------
- #
- # genbki.sh--
- #    shell script which generates .bki files from specially formatted .h
- #    files.  These .bki files are used to initialize the postgres template
- #    database.
- #
- # Portions Copyright (c) 1996-2009, PostgreSQL Global Development Group
- # Portions Copyright (c) 1994, Regents of the University of California
- #
- #
- # IDENTIFICATION
- #    $PostgreSQL$
- #
- # NOTES
- #    non-essential whitespace is removed from the generated file.
- #    if this is ever a problem, then the sed script at the very
- #    end can be changed into another awk script or something smarter.
- #
- #-------------------------------------------------------------------------
- 
- : ${AWK='awk'}
- 
- CMDNAME=`basename $0`
- 
- INCLUDE_DIRS=
- OUTPUT_PREFIX=
- INFILES=
- major_version=
- 
- #
- # Process command line switches.
- #
- while [ $# -gt 0 ]
- do
-     case $1 in
-         -I)
-             INCLUDE_DIRS="$INCLUDE_DIRS $2"
-             shift;;
-         -I*)
-             arg=`echo $1 | sed -e 's/^-I//'`
-             INCLUDE_DIRS="$INCLUDE_DIRS $arg"
-             ;;
-         -o)
-             OUTPUT_PREFIX="$2"
-             shift;;
-         -o*)
-             OUTPUT_PREFIX=`echo $1 | sed -e 's/^-o//'`
-             ;;
-         --set-version=*)
-             arg=`expr x"$1" : x"--set-version=\(.*\)"`
-             major_version=`expr x"$arg" : x'\([0-9][0-9]*\.[0-9][0-9]*\)'`
-             ;;
-         --help)
-             echo "$CMDNAME generates system catalog bootstrapping files."
-             echo
-             echo "Usage:"
-             echo "  $CMDNAME [ -I dir ] --set-version=VERSION -o prefix files..."
-             echo
-             echo "Options:"
-             echo "  -I  path to include files"
-             echo "  -o  prefix of output files"
-             echo "  --set-version  PostgreSQL version number for initdb cross-check"
-             echo
-             echo "The environment variable AWK determines which Awk program"
-             echo "to use. The default is \`awk'."
-             echo
-             echo "Report bugs to <pgsql-bugs@postgresql.org>."
-             exit 0
-             ;;
-         -*)
-             echo "$CMDNAME: invalid option: $1"
-             exit 1
-             ;;
-         *)
-             INFILES="$INFILES $1"
-             ;;
-     esac
-     shift
- done
- 
- if [ x"$INFILES" = x"" ] ; then
-     echo "$CMDNAME: no input files" 1>&2
-     exit 1
- fi
- 
- if [ x"$OUTPUT_PREFIX" = x"" ] ; then
-     echo "$CMDNAME: no output prefix specified" 1>&2
-     exit 1
- fi
- 
- if [ x"$INCLUDE_DIRS" = x"" ] ; then
-     echo "$CMDNAME: path to include directory unknown" 1>&2
-     exit 1
- fi
- 
- if [ x"$major_version" = x"" ] ; then
-     echo "$CMDNAME: invalid or no version number specified" 1>&2
-     exit 1
- fi
- 
- 
- TMPFILE="genbkitmp$$.c"
- 
- trap "rm -f $TMPFILE ${OUTPUT_PREFIX}.bki.$$ ${OUTPUT_PREFIX}.description.$$ ${OUTPUT_PREFIX}.shdescription.$$" 0 1 2 3 15
- 
- 
- # CAUTION: be wary about what symbols you substitute into the .bki file here!
- # It's okay to substitute things that are expected to be really constant
- # within a given Postgres release, such as fixed OIDs.  Do not substitute
- # anything that could depend on platform or configuration.  (The right place
- # to handle those sorts of things is in initdb.c's bootstrap_template1().)
- 
- # Get BOOTSTRAP_SUPERUSERID from catalog/pg_authid.h
- for dir in $INCLUDE_DIRS; do
-     if [ -f "$dir/catalog/pg_authid.h" ]; then
-         BOOTSTRAP_SUPERUSERID=`grep '^#define[ 	]*BOOTSTRAP_SUPERUSERID' $dir/catalog/pg_authid.h | $AWK '{ print $3 }'`
-         break
-     fi
- done
- 
- # Get PG_CATALOG_NAMESPACE from catalog/pg_namespace.h
- for dir in $INCLUDE_DIRS; do
-     if [ -f "$dir/catalog/pg_namespace.h" ]; then
-         PG_CATALOG_NAMESPACE=`grep '^#define[ 	]*PG_CATALOG_NAMESPACE' $dir/catalog/pg_namespace.h | $AWK '{ print $3 }'`
-         break
-     fi
- done
- 
- touch ${OUTPUT_PREFIX}.description.$$
- touch ${OUTPUT_PREFIX}.shdescription.$$
- 
- # ----------------
- # 	Strip comments and other trash from .h
- #
- #	Put multi-line start/end comments on a separate line
- #
- #	Rename datatypes that have different names in .h files than in SQL
- #
- #	Substitute values of configuration constants
- # ----------------
- #
- cat $INFILES | \
- sed -e 's;/\*.*\*/;;g' \
-     -e 's;/\*;\
- /*\
- ;g' \
-     -e 's;\*/;\
- */\
- ;g' | # we must run a new sed here to see the newlines we added
- sed -e "s/;[ 	]*$//g" \
-     -e "s/^[ 	]*//" \
-     -e "s/[ 	]Oid/ oid/g" \
-     -e "s/^Oid/oid/g" \
-     -e "s/(Oid/(oid/g" \
-     -e "s/[ 	]NameData/ name/g" \
-     -e "s/^NameData/name/g" \
-     -e "s/(NameData/(name/g" \
-     -e "s/[ 	]TransactionId/ xid/g" \
-     -e "s/^TransactionId/xid/g" \
-     -e "s/(TransactionId/(xid/g" \
-     -e "s/PGUID/$BOOTSTRAP_SUPERUSERID/g" \
-     -e "s/PGNSP/$PG_CATALOG_NAMESPACE/g" \
- | $AWK '
- # ----------------
- #	now use awk to process remaining .h file..
- #
- #	nc is the number of catalogs
- #	inside is a variable set to 1 when we are scanning the
- #	   contents of a catalog definition.
- #	reln_open is a flag indicating when we are processing DATA lines.
- #	   (i.e. have a relation open and need to close it)
- #	oid is the most recently seen oid, or 0 if none in the last DATA line.
- # ----------------
- BEGIN {
- 	inside = 0;
- 	bootstrap = "";
- 	shared_relation = "";
- 	without_oids = "";
- 	nc = 0;
- 	reln_open = 0;
- 	comment_level = 0;
- 	oid = 0;
- }
- 
- # ----------------
- # Anything in a /* .. */ block should be ignored.
- # Blank lines also go.
- # Note that any /* */ comment on a line by itself was removed from the line
- # by the sed above.
- # ----------------
- /^\/\*/           { comment_level += 1; next; }
- /^\*\//           { comment_level -= 1; next; }
- comment_level > 0 { next; }
- 
- /^[ 	]*$/      { next; }
- 
- # ----------------
- #	DATA() statements are basically passed right through after
- #	stripping off the DATA( and the ) on the end.
- #	Remember the OID for use by DESCR() and SHDESCR().
- # ----------------
- /^DATA\(/ {
- 	data = substr($0, 6, length($0) - 6);
- 	oid = 0;
- 	nf = split(data, datafields);
- 	if (nf >= 4 && datafields[1] == "insert" && datafields[2] == "OID" && datafields[3] == "=")
- 	{
- 		oid = datafields[4];
- 	}
- 	print data;
- 	next;
- }
- 
- /^DESCR\(/ {
- 	if (oid != 0)
- 	{
- 		data = substr($0, 8, length($0) - 9);
- 		if (data != "")
- 			printf "%d\t%s\t0\t%s\n", oid, catalog, data >>descriptionfile;
- 	}
- 	next;
- }
- 
- /^SHDESCR\(/ {
- 	if (oid != 0)
- 	{
- 		data = substr($0, 10, length($0) - 11);
- 		if (data != "")
- 			printf "%d\t%s\t%s\n", oid, catalog, data >>shdescriptionfile;
- 	}
- 	next;
- }
- 
- /^DECLARE_INDEX\(/ {
- # ----
- #  end any prior catalog data insertions before starting a define index
- # ----
- 	if (reln_open == 1) {
- 		print "close " catalog;
- 		reln_open = 0;
- 	}
- 
- 	data = substr($0, 15, length($0) - 15);
- 	pos = index(data, ",");
- 	iname = substr(data, 1, pos-1);
- 	data = substr(data, pos+1, length(data)-pos);
- 	pos = index(data, ",");
- 	oid = substr(data, 1, pos-1);
- 	data = substr(data, pos+1, length(data)-pos);
- 
- 	print "declare index " iname " " oid " " data
- }
- 
- /^DECLARE_UNIQUE_INDEX\(/ {
- # ----
- #  end any prior catalog data insertions before starting a define unique index
- # ----
- 	if (reln_open == 1) {
- 		print "close " catalog;
- 		reln_open = 0;
- 	}
- 
- 	data = substr($0, 22, length($0) - 22);
- 	pos = index(data, ",");
- 	iname = substr(data, 1, pos-1);
- 	data = substr(data, pos+1, length(data)-pos);
- 	pos = index(data, ",");
- 	oid = substr(data, 1, pos-1);
- 	data = substr(data, pos+1, length(data)-pos);
- 
- 	print "declare unique index " iname " " oid " " data
- }
- 
- /^DECLARE_TOAST\(/ {
- # ----
- #  end any prior catalog data insertions before starting a define toast
- # ----
- 	if (reln_open == 1) {
- 		print "close " catalog;
- 		reln_open = 0;
- 	}
- 
- 	data = substr($0, 15, length($0) - 15);
- 	pos = index(data, ",");
- 	tname = substr(data, 1, pos-1);
- 	data = substr(data, pos+1, length(data)-pos);
- 	pos = index(data, ",");
- 	toastoid = substr(data, 1, pos-1);
- 	data = substr(data, pos+1, length(data)-pos);
- 	# previous commands already removed the trailing );
- 	indexoid = data;
- 
- 	print "declare toast " toastoid " " indexoid " on " tname
- }
- 
- /^BUILD_INDICES/	{ print "build indices"; }
- 	
- # ----------------
- #	CATALOG() definitions take some more work.
- # ----------------
- /^CATALOG\(/ { 
- # ----
- #  end any prior catalog data insertions before starting a new one..
- # ----
- 	if (reln_open == 1) {
- 		print "close " catalog;
- 		reln_open = 0;
- 	}
- 
- # ----
- #  get the name and properties of the new catalog
- # ----
- 	pos = index($1,")");
- 	catalogandoid = substr($1,9,pos-9);
- 	pos = index(catalogandoid, ",");
- 	catalog = substr(catalogandoid, 1, pos-1);
- 	oid = substr(catalogandoid, pos+1, length(catalogandoid)-pos);
- 
- 	if ($0 ~ /BKI_BOOTSTRAP/) {
- 		bootstrap = "bootstrap ";
- 	}
- 	if ($0 ~ /BKI_SHARED_RELATION/) {
- 		shared_relation = "shared_relation ";
- 	}
- 	if ($0 ~ /BKI_WITHOUT_OIDS/) {
- 		without_oids = "without_oids ";
- 	}
- 
-         i = 1;
- 	inside = 1;
-         nc++;
- 	next;
- }
- 
- # ----------------
- #	process the columns of the catalog definition
- #
- #	attname[ x ] contains the attribute name for attribute x
- #	atttype[ x ] contains the attribute type fot attribute x
- # ----------------
- inside == 1 {
- # ----
- #  ignore a leading brace line..
- # ----
-         if ($1 ~ /\{/)
- 		next;
- 
- # ----
- #  if this is the last line, then output the bki catalog stuff.
- # ----
- 	if ($1 ~ /}/) {
- 		print "create " bootstrap shared_relation without_oids catalog " " oid;
- 		print "\t(";
- 
- 		for (j=1; j<i-1; j++) {
- 			print "\t " attname[ j ] " = " atttype[ j ] " ,";
- 		}
- 		print "\t " attname[ j ] " = " atttype[ j ] ;
- 		print "\t)";
- 
- 		if (bootstrap == "") {
- 			print "open " catalog;
- 		}
- 
- 		i = 1;
- 		reln_open = 1;
- 		inside = 0;
- 		bootstrap = "";
- 		shared_relation = "";
- 		without_oids = "";
- 		next;
- 	}
- 
- # ----
- #  we are inside the catalog definition, so keep sucking up
- #  attribute names and types
- # ----
- 	if ($2 ~ /\[.*\]/) {			# array attribute
- 		idlen = index($2,"[") - 1;
- 		atttype[ i ] = $1 "[]";		# variable-length only..
- 		attname[ i ] = substr($2,1,idlen);
- 	} else {
- 		atttype[ i ] = $1;
- 		attname[ i ] = $2;
- 	}
- 	i++;
- 	next;
- }
- 
- END {
- 	if (reln_open == 1) {
- 		print "close " catalog;
- 		reln_open = 0;
- 	}
- }
- ' "descriptionfile=${OUTPUT_PREFIX}.description.$$" "shdescriptionfile=${OUTPUT_PREFIX}.shdescription.$$" > $TMPFILE || exit
- 
- echo "# PostgreSQL $major_version" >${OUTPUT_PREFIX}.bki.$$
- 
- sed -e '/^[ 	]*$/d' \
-     -e 's/[ 	][ 	]*/ /g' $TMPFILE >>${OUTPUT_PREFIX}.bki.$$ || exit
- 
- #
- # Sanity check: if one of the sed/awk/etc commands fails, we'll probably
- # end up with a .bki file that is empty or just a few lines.  Cross-check
- # that the files are of reasonable size.  The numbers here are arbitrary,
- # but are much smaller than the actual expected sizes as of Postgres 7.2.
- #
- if [ `wc -c < ${OUTPUT_PREFIX}.bki.$$` -lt 100000 ]; then
-     echo "$CMDNAME: something seems to be wrong with the .bki file" >&2
-     exit 1
- fi
- if [ `wc -c < ${OUTPUT_PREFIX}.description.$$` -lt 10000 ]; then
-     echo "$CMDNAME: something seems to be wrong with the .description file" >&2
-     exit 1
- fi
- if [ `wc -c < ${OUTPUT_PREFIX}.shdescription.$$` -lt 10 ]; then
-     echo "$CMDNAME: something seems to be wrong with the .shdescription file" >&2
-     exit 1
- fi
- 
- # Looks good, commit ...
- 
- mv ${OUTPUT_PREFIX}.bki.$$ ${OUTPUT_PREFIX}.bki || exit
- mv ${OUTPUT_PREFIX}.description.$$ ${OUTPUT_PREFIX}.description || exit
- mv ${OUTPUT_PREFIX}.shdescription.$$ ${OUTPUT_PREFIX}.shdescription || exit
- 
- exit 0
--- 0 ----
*** a/src/backend/utils/.cvsignore
--- /dev/null
***************
*** 1,2 ****
- fmgrtab.c
- fmgroids.h
--- 0 ----
*** a/src/backend/utils/Gen_fmgrtab.pl
--- /dev/null
***************
*** 1,194 ****
- #! /usr/bin/perl -w
- #-------------------------------------------------------------------------
- #
- # Gen_fmgrtab.pl
- #    Perl equivalent of Gen_fmgrtab.sh
- #
- # Usage: perl Gen_fmgrtab.pl path-to-pg_proc.h
- #
- # The reason for implementing this functionality twice is that we don't
- # require people to have perl to build from a tarball, but on the other
- # hand Windows can't deal with shell scripts.
- #
- # Portions Copyright (c) 1996-2009, PostgreSQL Global Development Group
- # Portions Copyright (c) 1994, Regents of the University of California
- #
- #
- # IDENTIFICATION
- #    $PostgreSQL$
- #
- #-------------------------------------------------------------------------
- 
- use strict;
- use warnings;
- 
- # Collect arguments
- my $infile = shift;
- defined($infile) || die "$0: missing required argument: pg_proc.h\n";
- 
- # Note: see Gen_fmgrtab.sh for detailed commentary on what this is doing
- 
- # Collect column numbers for pg_proc columns we need
- my ($proname, $prolang, $proisstrict, $proretset, $pronargs, $prosrc);
- 
- open(I, $infile) || die "Could not open $infile: $!";
- while (<I>)
- {
-     if (m/#define Anum_pg_proc_proname\s+(\d+)/) {
- 	$proname = $1;
-     }
-     if (m/#define Anum_pg_proc_prolang\s+(\d+)/) {
- 	$prolang = $1;
-     }
-     if (m/#define Anum_pg_proc_proisstrict\s+(\d+)/) {
- 	$proisstrict = $1;
-     }
-     if (m/#define Anum_pg_proc_proretset\s+(\d+)/) {
- 	$proretset = $1;
-     }
-     if (m/#define Anum_pg_proc_pronargs\s+(\d+)/) {
- 	$pronargs = $1;
-     }
-     if (m/#define Anum_pg_proc_prosrc\s+(\d+)/) {
- 	$prosrc = $1;
-     }
- }
- close(I);
- 
- # Collect the raw data
- my @fmgr = ();
- 
- open(I, $infile) || die "Could not open $infile: $!";
- while (<I>)
- {
-     next unless (/^DATA/);
-     s/^[^O]*OID[^=]*=[ \t]*//;
-     s/\(//;
-     s/"[^"]*"/"xxx"/g;
-     my @p = split;
-     next if ($p[$prolang] ne "12");
-     push @fmgr,
-       {
-         oid     => $p[0],
-         proname => $p[$proname],
-         strict  => $p[$proisstrict],
-         retset  => $p[$proretset],
-         nargs   => $p[$pronargs],
-         prosrc  => $p[$prosrc],
-       };
- }
- close(I);
- 
- # Emit headers for both files
- open(H, '>', "$$-fmgroids.h") || die "Could not open $$-fmgroids.h: $!";
- print H 
- qq|/*-------------------------------------------------------------------------
-  *
-  * fmgroids.h
-  *    Macros that define the OIDs of built-in functions.
-  *
-  * These macros can be used to avoid a catalog lookup when a specific
-  * fmgr-callable function needs to be referenced.
-  *
-  * Portions Copyright (c) 1996-2009, PostgreSQL Global Development Group
-  * Portions Copyright (c) 1994, Regents of the University of California
-  *
-  * NOTES
-  *	******************************
-  *	*** DO NOT EDIT THIS FILE! ***
-  *	******************************
-  *
-  *	It has been GENERATED by $0
-  *	from $infile
-  *
-  *-------------------------------------------------------------------------
-  */
- #ifndef FMGROIDS_H
- #define FMGROIDS_H
- 
- /*
-  *	Constant macros for the OIDs of entries in pg_proc.
-  *
-  *	NOTE: macros are named after the prosrc value, ie the actual C name
-  *	of the implementing function, not the proname which may be overloaded.
-  *	For example, we want to be able to assign different macro names to both
-  *	char_text() and name_text() even though these both appear with proname
-  *	'text'.  If the same C function appears in more than one pg_proc entry,
-  *	its equivalent macro will be defined with the lowest OID among those
-  *	entries.
-  */
- |;
- 
- open(T, '>', "$$-fmgrtab.c") || die "Could not open $$-fmgrtab.c: $!";
- print T
- qq|/*-------------------------------------------------------------------------
-  *
-  * fmgrtab.c
-  *    The function manager's table of internal functions.
-  *
-  * Portions Copyright (c) 1996-2009, PostgreSQL Global Development Group
-  * Portions Copyright (c) 1994, Regents of the University of California
-  *
-  * NOTES
-  *
-  *	******************************
-  *	*** DO NOT EDIT THIS FILE! ***
-  *	******************************
-  *
-  *	It has been GENERATED by $0
-  *	from $infile
-  *
-  *-------------------------------------------------------------------------
-  */
- 
- #include "postgres.h"
- 
- #include "utils/fmgrtab.h"
- 
- |;
- 
- # Emit #define's and extern's -- only one per prosrc value
- my %seenit;
- foreach my $s (sort {$a->{oid} <=> $b->{oid}} @fmgr)
- {
-     next if $seenit{$s->{prosrc}};
-     $seenit{$s->{prosrc}} = 1;
-     print H "#define F_" . uc $s->{prosrc} . " $s->{oid}\n";
-     print T "extern Datum $s->{prosrc} (PG_FUNCTION_ARGS);\n";
- }
- 
- # Create the fmgr_builtins table
- print T "\nconst FmgrBuiltin fmgr_builtins[] = {\n";
- my %bmap;
- $bmap{'t'} = 'true';
- $bmap{'f'} = 'false';
- foreach my $s (sort {$a->{oid} <=> $b->{oid}} @fmgr)
- {
-     print T
- 	"  { $s->{oid}, \"$s->{prosrc}\", $s->{nargs}, $bmap{$s->{strict}}, $bmap{$s->{retset}}, $s->{prosrc} },\n";
- }
- 
- # And add the file footers.
- print H "\n#endif /* FMGROIDS_H */\n";
- close(H);
- 
- print T
- qq|  /* dummy entry is easier than getting rid of comma after last real one */
-   /* (not that there has ever been anything wrong with *having* a
-      comma after the last field in an array initializer) */
-   { 0, NULL, 0, false, false, NULL }
- };
- 
- /* Note fmgr_nbuiltins excludes the dummy entry */
- const int fmgr_nbuiltins = (sizeof(fmgr_builtins) / sizeof(FmgrBuiltin)) - 1;
- |;
- 
- close(T);
- 
- # Finally, rename the completed files into place.
- rename "$$-fmgroids.h", "fmgroids.h"
-     || die "Could not rename $$-fmgroids.h to fmgroids.h: $!";
- rename "$$-fmgrtab.c", "fmgrtab.c"
-     || die "Could not rename $$-fmgrtab.c to fmgrtab.c: $!";
- 
- exit 0;
--- 0 ----
*** a/src/backend/utils/Gen_fmgrtab.sh
--- /dev/null
***************
*** 1,253 ****
- #! /bin/sh
- #-------------------------------------------------------------------------
- #
- # Gen_fmgrtab.sh
- #    shell script to generate fmgroids.h and fmgrtab.c from pg_proc.h
- #
- # NOTE: if you change this, you need to fix Gen_fmgrtab.pl too!
- #
- # Portions Copyright (c) 1996-2009, PostgreSQL Global Development Group
- # Portions Copyright (c) 1994, Regents of the University of California
- #
- #
- # IDENTIFICATION
- #    $PostgreSQL$
- #
- #-------------------------------------------------------------------------
- 
- CMDNAME=`basename $0`
- 
- if [ x"$AWK" = x"" ]; then
- 	AWK=awk
- fi
- 
- cleanup(){
-     [ x"$noclean" != x"t" ] && rm -f "$SORTEDFILE" "$$-$OIDSFILE" "$$-$TABLEFILE"
- }
- 
- noclean=
- 
- #
- # Process command line switches.
- #
- while [ $# -gt 0 ]
- do
-     case $1 in
-         --noclean)
-             noclean=t
-             ;;
-         --help)
-             echo "$CMDNAME generates fmgroids.h and fmgrtab.c from pg_proc.h."
-             echo
-             echo "Usage:"
-             echo "  $CMDNAME inputfile"
-             echo
-             echo "The environment variable AWK determines which Awk program"
-             echo "to use. The default is \`awk'."
-             echo
-             echo "Report bugs to <pgsql-bugs@postgresql.org>."
-             exit 0
-             ;;
-         -*)
-             echo "$CMDNAME: invalid option: $1"
-             exit 1
-             ;;
-         *)
-             INFILE=$1
-             ;;
-     esac
-     shift
- done
- 
- 
- if [ x"$INFILE" = x ] ; then
-     echo "$CMDNAME: no input file"
-     exit 1
- fi
- 
- SORTEDFILE="$$-fmgr.data"
- OIDSFILE=fmgroids.h
- TABLEFILE=fmgrtab.c
- 
- 
- trap 'echo "Caught signal." ; cleanup ; exit 1' 1 2 15
- 
- #
- # Collect the column numbers of the pg_proc columns we need.  Because we will
- # be looking at data that includes the OID as the first column, add one to
- # each column number.
- #
- proname=`egrep '^#define Anum_pg_proc_proname[ 	]' $INFILE | $AWK '{print $3+1}'`
- prolang=`egrep '^#define Anum_pg_proc_prolang[ 	]' $INFILE | $AWK '{print $3+1}'`
- proisstrict=`egrep '^#define Anum_pg_proc_proisstrict[ 	]' $INFILE | $AWK '{print $3+1}'`
- proretset=`egrep '^#define Anum_pg_proc_proretset[ 	]' $INFILE | $AWK '{print $3+1}'`
- pronargs=`egrep '^#define Anum_pg_proc_pronargs[ 	]' $INFILE | $AWK '{print $3+1}'`
- prosrc=`egrep '^#define Anum_pg_proc_prosrc[ 	]' $INFILE | $AWK '{print $3+1}'`
- 
- #
- # Generate the file containing raw pg_proc data.  We do three things here:
- # 1. Strip off the DATA macro call, leaving procedure OID as $1
- # and all the pg_proc field values as $2, $3, etc on each line.
- # 2. Fold quoted fields to simple "xxx".  We need this because such fields
- # may contain whitespace, which would confuse awk's counting of fields.
- # Fortunately, this script doesn't need to look at any fields that might
- # need quoting, so this simple hack is sufficient.
- # 3. Select out just the rows for internal-language procedures.
- #
- # Note assumption here that INTERNALlanguageId == 12.
- #
- egrep '^DATA' $INFILE | \
- sed 	-e 's/^[^O]*OID[^=]*=[ 	]*//' \
- 	-e 's/(//' \
- 	-e 's/"[^"]*"/"xxx"/g' | \
- $AWK "\$$prolang == \"12\" { print }" | \
- sort -n > $SORTEDFILE
- 
- if [ $? -ne 0 ]; then
-     cleanup
-     echo "$CMDNAME failed"
-     exit 1
- fi
- 
- 
- cpp_define=`echo $OIDSFILE | tr abcdefghijklmnopqrstuvwxyz ABCDEFGHIJKLMNOPQRSTUVWXYZ | sed -e 's/[^ABCDEFGHIJKLMNOPQRSTUVWXYZ]/_/g'`
- 
- #
- # Generate fmgroids.h
- #
- cat > "$$-$OIDSFILE" <<FuNkYfMgRsTuFf
- /*-------------------------------------------------------------------------
-  *
-  * $OIDSFILE
-  *    Macros that define the OIDs of built-in functions.
-  *
-  * These macros can be used to avoid a catalog lookup when a specific
-  * fmgr-callable function needs to be referenced.
-  *
-  * Portions Copyright (c) 1996-2009, PostgreSQL Global Development Group
-  * Portions Copyright (c) 1994, Regents of the University of California
-  *
-  * NOTES
-  *	******************************
-  *	*** DO NOT EDIT THIS FILE! ***
-  *	******************************
-  *
-  *	It has been GENERATED by $CMDNAME
-  *	from $INFILE
-  *
-  *-------------------------------------------------------------------------
-  */
- #ifndef $cpp_define
- #define $cpp_define
- 
- /*
-  *	Constant macros for the OIDs of entries in pg_proc.
-  *
-  *	NOTE: macros are named after the prosrc value, ie the actual C name
-  *	of the implementing function, not the proname which may be overloaded.
-  *	For example, we want to be able to assign different macro names to both
-  *	char_text() and name_text() even though these both appear with proname
-  *	'text'.  If the same C function appears in more than one pg_proc entry,
-  *	its equivalent macro will be defined with the lowest OID among those
-  *	entries.
-  */
- FuNkYfMgRsTuFf
- 
- tr 'abcdefghijklmnopqrstuvwxyz' 'ABCDEFGHIJKLMNOPQRSTUVWXYZ' < $SORTEDFILE | \
- $AWK "{ if (seenit[\$$prosrc]++ == 0)
- 	printf \"#define F_%s %s\\n\", \$$prosrc, \$1; }" >> "$$-$OIDSFILE"
- 
- if [ $? -ne 0 ]; then
-     cleanup
-     echo "$CMDNAME failed"
-     exit 1
- fi
- 
- cat >> "$$-$OIDSFILE" <<FuNkYfMgRsTuFf
- 
- #endif /* $cpp_define */
- FuNkYfMgRsTuFf
- 
- #
- # Generate fmgr's built-in-function table.
- #
- # Print out the function declarations, then the table that refers to them.
- #
- cat > "$$-$TABLEFILE" <<FuNkYfMgRtAbStUfF
- /*-------------------------------------------------------------------------
-  *
-  * $TABLEFILE
-  *    The function manager's table of internal functions.
-  *
-  * Portions Copyright (c) 1996-2009, PostgreSQL Global Development Group
-  * Portions Copyright (c) 1994, Regents of the University of California
-  *
-  * NOTES
-  *
-  *	******************************
-  *	*** DO NOT EDIT THIS FILE! ***
-  *	******************************
-  *
-  *	It has been GENERATED by $CMDNAME
-  *	from $INFILE
-  *
-  *-------------------------------------------------------------------------
-  */
- 
- #include "postgres.h"
- 
- #include "utils/fmgrtab.h"
- 
- FuNkYfMgRtAbStUfF
- 
- $AWK "{ if (seenit[\$$prosrc]++ == 0)
- 	print \"extern Datum\", \$$prosrc, \"(PG_FUNCTION_ARGS);\"; }" $SORTEDFILE >> "$$-$TABLEFILE"
- 
- if [ $? -ne 0 ]; then
-     cleanup
-     echo "$CMDNAME failed"
-     exit 1
- fi
- 
- 
- cat >> "$$-$TABLEFILE" <<FuNkYfMgRtAbStUfF
- 
- const FmgrBuiltin fmgr_builtins[] = {
- FuNkYfMgRtAbStUfF
- 
- # Note: using awk arrays to translate from pg_proc values to fmgrtab values
- # may seem tedious, but avoid the temptation to write a quick x?y:z
- # conditional expression instead.  Not all awks have conditional expressions.
- 
- $AWK "BEGIN {
-     Bool[\"t\"] = \"true\";
-     Bool[\"f\"] = \"false\";
- }
- { printf (\"  { %d, \\\"%s\\\", %d, %s, %s, %s },\\n\"),
- 	\$1, \$$prosrc, \$$pronargs, Bool[\$$proisstrict], Bool[\$$proretset], \$$prosrc ;
- }" $SORTEDFILE >> "$$-$TABLEFILE"
- 
- if [ $? -ne 0 ]; then
-     cleanup
-     echo "$CMDNAME failed"
-     exit 1
- fi
- 
- cat >> "$$-$TABLEFILE" <<FuNkYfMgRtAbStUfF
-   /* dummy entry is easier than getting rid of comma after last real one */
-   /* (not that there has ever been anything wrong with *having* a
-      comma after the last field in an array initializer) */
-   { 0, NULL, 0, false, false, NULL }
- };
- 
- /* Note fmgr_nbuiltins excludes the dummy entry */
- const int fmgr_nbuiltins = (sizeof(fmgr_builtins) / sizeof(FmgrBuiltin)) - 1;
- FuNkYfMgRtAbStUfF
- 
- # We use the temporary files to avoid problems with concurrent runs
- # (which can happen during parallel make).
- mv "$$-$OIDSFILE" $OIDSFILE
- mv "$$-$TABLEFILE" $TABLEFILE
- 
- cleanup
- exit 0
--- 0 ----
*** a/src/backend/utils/Makefile
--- b/src/backend/utils/Makefile
***************
*** 8,24 **** subdir = src/backend/utils
  top_builddir = ../../..
  include $(top_builddir)/src/Makefile.global
  
- OBJS        = fmgrtab.o
  SUBDIRS     = adt cache error fmgr hash init mb misc mmgr resowner sort time
  
  include $(top_srcdir)/src/backend/common.mk
  
! all: fmgroids.h probes.h
! 
! $(SUBDIRS:%=%-recursive): fmgroids.h
! 
! fmgroids.h fmgrtab.c: Gen_fmgrtab.sh $(top_srcdir)/src/include/catalog/pg_proc.h
! 	AWK='$(AWK)' $(SHELL) $< $(top_srcdir)/src/include/catalog/pg_proc.h
  
  ifneq ($(enable_dtrace), yes)
  probes.h: Gen_dummy_probes.sed
--- 8,18 ----
  top_builddir = ../../..
  include $(top_builddir)/src/Makefile.global
  
  SUBDIRS     = adt cache error fmgr hash init mb misc mmgr resowner sort time
  
  include $(top_srcdir)/src/backend/common.mk
  
! all: probes.h
  
  ifneq ($(enable_dtrace), yes)
  probes.h: Gen_dummy_probes.sed
***************
*** 35,38 **** endif
  
  
  clean:
! 	rm -f fmgroids.h fmgrtab.c probes.h
--- 29,32 ----
  
  
  clean:
! 	rm -f probes.h
*** a/src/backend/utils/cache/relcache.c
--- b/src/backend/utils/cache/relcache.c
***************
*** 48,53 ****
--- 48,54 ----
  #include "catalog/pg_proc.h"
  #include "catalog/pg_rewrite.h"
  #include "catalog/pg_type.h"
+ #include "catalog/schemapg.h"
  #include "commands/trigger.h"
  #include "miscadmin.h"
  #include "optimizer/clauses.h"
*** a/src/include/catalog/pg_attribute.h
--- b/src/include/catalog/pg_attribute.h
***************
*** 219,254 **** typedef FormData_pg_attribute *Form_pg_attribute;
   *		pg_type
   * ----------------
   */
- #define Schema_pg_type \
- { 1247, {"typname"},	   19, -1, 0, NAMEDATALEN, 1, 0, -1, -1, false, 'p', 'c', true, false, false, true, 0, { 0 } }, \
- { 1247, {"typnamespace"},  26, -1, 0,	4,	2, 0, -1, -1, true, 'p', 'i', true, false, false, true, 0, { 0 } }, \
- { 1247, {"typowner"},	   26, -1, 0,	4,	3, 0, -1, -1, true, 'p', 'i', true, false, false, true, 0, { 0 } }, \
- { 1247, {"typlen"},		   21, -1, 0,	2,	4, 0, -1, -1, true, 'p', 's', true, false, false, true, 0, { 0 } }, \
- { 1247, {"typbyval"},	   16, -1, 0,	1,	5, 0, -1, -1, true, 'p', 'c', true, false, false, true, 0, { 0 } }, \
- { 1247, {"typtype"},	   18, -1, 0,	1,	6, 0, -1, -1, true, 'p', 'c', true, false, false, true, 0, { 0 } }, \
- { 1247, {"typcategory"},   18, -1, 0,	1,	7, 0, -1, -1, true, 'p', 'c', true, false, false, true, 0, { 0 } }, \
- { 1247, {"typispreferred"},16, -1, 0,	1,	8, 0, -1, -1, true, 'p', 'c', true, false, false, true, 0, { 0 } }, \
- { 1247, {"typisdefined"},  16, -1, 0,	1,	9, 0, -1, -1, true, 'p', 'c', true, false, false, true, 0, { 0 } }, \
- { 1247, {"typdelim"},	   18, -1, 0,	1, 10, 0, -1, -1, true, 'p', 'c', true, false, false, true, 0, { 0 } }, \
- { 1247, {"typrelid"},	   26, -1, 0,	4, 11, 0, -1, -1, true, 'p', 'i', true, false, false, true, 0, { 0 } }, \
- { 1247, {"typelem"},	   26, -1, 0,	4, 12, 0, -1, -1, true, 'p', 'i', true, false, false, true, 0, { 0 } }, \
- { 1247, {"typarray"},	   26, -1, 0,	4, 13, 0, -1, -1, true, 'p', 'i', true, false, false, true, 0, { 0 } }, \
- { 1247, {"typinput"},	   24, -1, 0,	4, 14, 0, -1, -1, true, 'p', 'i', true, false, false, true, 0, { 0 } }, \
- { 1247, {"typoutput"},	   24, -1, 0,	4, 15, 0, -1, -1, true, 'p', 'i', true, false, false, true, 0, { 0 } }, \
- { 1247, {"typreceive"},    24, -1, 0,	4, 16, 0, -1, -1, true, 'p', 'i', true, false, false, true, 0, { 0 } }, \
- { 1247, {"typsend"},	   24, -1, 0,	4, 17, 0, -1, -1, true, 'p', 'i', true, false, false, true, 0, { 0 } }, \
- { 1247, {"typmodin"},	   24, -1, 0,	4, 18, 0, -1, -1, true, 'p', 'i', true, false, false, true, 0, { 0 } }, \
- { 1247, {"typmodout"},	   24, -1, 0,	4, 19, 0, -1, -1, true, 'p', 'i', true, false, false, true, 0, { 0 } }, \
- { 1247, {"typanalyze"},    24, -1, 0,	4, 20, 0, -1, -1, true, 'p', 'i', true, false, false, true, 0, { 0 } }, \
- { 1247, {"typalign"},	   18, -1, 0,	1, 21, 0, -1, -1, true, 'p', 'c', true, false, false, true, 0, { 0 } }, \
- { 1247, {"typstorage"},    18, -1, 0,	1, 22, 0, -1, -1, true, 'p', 'c', true, false, false, true, 0, { 0 } }, \
- { 1247, {"typnotnull"},    16, -1, 0,	1, 23, 0, -1, -1, true, 'p', 'c', true, false, false, true, 0, { 0 } }, \
- { 1247, {"typbasetype"},   26, -1, 0,	4, 24, 0, -1, -1, true, 'p', 'i', true, false, false, true, 0, { 0 } }, \
- { 1247, {"typtypmod"},	   23, -1, 0,	4, 25, 0, -1, -1, true, 'p', 'i', true, false, false, true, 0, { 0 } }, \
- { 1247, {"typndims"},	   23, -1, 0,	4, 26, 0, -1, -1, true, 'p', 'i', true, false, false, true, 0, { 0 } }, \
- { 1247, {"typdefaultbin"}, 25, -1, 0, -1, 27, 0, -1, -1, false, 'x', 'i', false, false, false, true, 0, { 0 } }, \
- { 1247, {"typdefault"},    25, -1, 0, -1, 28, 0, -1, -1, false, 'x', 'i', false, false, false, true, 0, { 0 } }
- 
  DATA(insert ( 1247 typname			19 -1 0 NAMEDATALEN	1 0 -1 -1 f p c t f f t 0 _null_));
  DATA(insert ( 1247 typnamespace		26 -1 0 4   2 0 -1 -1 t p i t f f t 0 _null_));
  DATA(insert ( 1247 typowner			26 -1 0 4   3 0 -1 -1 t p i t f f t 0 _null_));
--- 219,224 ----
***************
*** 289,321 **** DATA(insert ( 1247 tableoid			26 0 0  4  -7 0 -1 -1 t p i t f f t 0 _null_));
   *		pg_proc
   * ----------------
   */
- #define Schema_pg_proc \
- { 1255, {"proname"},			19, -1, 0, NAMEDATALEN,  1, 0, -1, -1, false, 'p', 'c', true, false, false, true, 0, { 0 } }, \
- { 1255, {"pronamespace"},		26, -1, 0, 4,	2, 0, -1, -1, true, 'p', 'i', true, false, false, true, 0, { 0 } }, \
- { 1255, {"proowner"},			26, -1, 0, 4,	3, 0, -1, -1, true, 'p', 'i', true, false, false, true, 0, { 0 } }, \
- { 1255, {"prolang"},			26, -1, 0, 4,	4, 0, -1, -1, true, 'p', 'i', true, false, false, true, 0, { 0 } }, \
- { 1255, {"procost"},		   700, -1, 0, 4,	5, 0, -1, -1, FLOAT4PASSBYVAL, 'p', 'i', true, false, false, true, 0, { 0 } }, \
- { 1255, {"prorows"},		   700, -1, 0, 4,	6, 0, -1, -1, FLOAT4PASSBYVAL, 'p', 'i', true, false, false, true, 0, { 0 } }, \
- { 1255, {"provariadic"},		26, -1, 0, 4,	7, 0, -1, -1, true, 'p', 'i', true, false, false, true, 0, { 0 } }, \
- { 1255, {"proisagg"},			16, -1, 0, 1,	8, 0, -1, -1, true, 'p', 'c', true, false, false, true, 0, { 0 } }, \
- { 1255, {"proiswindow"},		16, -1, 0, 1,	9, 0, -1, -1, true, 'p', 'c', true, false, false, true, 0, { 0 } }, \
- { 1255, {"prosecdef"},			16, -1, 0, 1, 10, 0, -1, -1, true, 'p', 'c', true, false, false, true, 0, { 0 } }, \
- { 1255, {"proisstrict"},		16, -1, 0, 1, 11, 0, -1, -1, true, 'p', 'c', true, false, false, true, 0, { 0 } }, \
- { 1255, {"proretset"},			16, -1, 0, 1, 12, 0, -1, -1, true, 'p', 'c', true, false, false, true, 0, { 0 } }, \
- { 1255, {"provolatile"},		18, -1, 0, 1, 13, 0, -1, -1, true, 'p', 'c', true, false, false, true, 0, { 0 } }, \
- { 1255, {"pronargs"},			21, -1, 0, 2, 14, 0, -1, -1, true, 'p', 's', true, false, false, true, 0, { 0 } }, \
- { 1255, {"pronargdefaults"},	21, -1, 0, 2, 15, 0, -1, -1, true, 'p', 's', true, false, false, true, 0, { 0 } }, \
- { 1255, {"prorettype"},			26, -1, 0, 4, 16, 0, -1, -1, true, 'p', 'i', true, false, false, true, 0, { 0 } }, \
- { 1255, {"proargtypes"},		30, -1, 0, -1, 17, 1, -1, -1, false, 'p', 'i', true, false, false, true, 0, { 0 } }, \
- { 1255, {"proallargtypes"},   1028, -1, 0, -1, 18, 1, -1, -1, false, 'x', 'i', false, false, false, true, 0, { 0 } }, \
- { 1255, {"proargmodes"},	  1002, -1, 0, -1, 19, 1, -1, -1, false, 'x', 'i', false, false, false, true, 0, { 0 } }, \
- { 1255, {"proargnames"},	  1009, -1, 0, -1, 20, 1, -1, -1, false, 'x', 'i', false, false, false, true, 0, { 0 } }, \
- { 1255, {"proargdefaults"},		25, -1, 0, -1, 21, 0, -1, -1, false, 'x', 'i', false, false, false, true, 0, { 0 } }, \
- { 1255, {"prosrc"},				25, -1, 0, -1, 22, 0, -1, -1, false, 'x', 'i', false, false, false, true, 0, { 0 } }, \
- { 1255, {"probin"},				17, -1, 0, -1, 23, 0, -1, -1, false, 'x', 'i', false, false, false, true, 0, { 0 } }, \
- { 1255, {"proconfig"},		  1009, -1, 0, -1, 24, 1, -1, -1, false, 'x', 'i', false, false, false, true, 0, { 0 } }, \
- { 1255, {"proacl"},			  1034, -1, 0, -1, 25, 1, -1, -1, false, 'x', 'i', false, false, false, true, 0, { 0 } }
- 
  DATA(insert ( 1255 proname			19 -1 0 NAMEDATALEN	1 0 -1 -1 f p c t f f t 0 _null_));
  DATA(insert ( 1255 pronamespace		26 -1 0 4   2 0 -1 -1 t p i t f f t 0 _null_));
  DATA(insert ( 1255 proowner			26 -1 0 4   3 0 -1 -1 t p i t f f t 0 _null_));
--- 259,264 ----
***************
*** 353,379 **** DATA(insert ( 1255 tableoid			26 0 0  4  -7 0 -1 -1 t p i t f f t 0 _null_));
   *		pg_attribute
   * ----------------
   */
- #define Schema_pg_attribute \
- { 1249, {"attrelid"},	  26, -1, 0,	4,	1, 0, -1, -1, true, 'p', 'i', true, false, false, true, 0, { 0 } }, \
- { 1249, {"attname"},	  19, -1, 0, NAMEDATALEN,	2, 0, -1, -1, false, 'p', 'c', true, false, false, true, 0, { 0 } }, \
- { 1249, {"atttypid"},	  26, -1, 0,	4,	3, 0, -1, -1, true, 'p', 'i', true, false, false, true, 0, { 0 } }, \
- { 1249, {"attstattarget"}, 23, -1, 0,	4,	4, 0, -1, -1, true, 'p', 'i', true, false, false, true, 0, { 0 } }, \
- { 1249, {"attdistinct"}, 700, -1, 0,	4,	5, 0, -1, -1, FLOAT4PASSBYVAL, 'p', 'i', true, false, false, true, 0, { 0 } }, \
- { 1249, {"attlen"},		  21, -1, 0,	2,	6, 0, -1, -1, true, 'p', 's', true, false, false, true, 0, { 0 } }, \
- { 1249, {"attnum"},		  21, -1, 0,	2,	7, 0, -1, -1, true, 'p', 's', true, false, false, true, 0, { 0 } }, \
- { 1249, {"attndims"},	  23, -1, 0,	4,	8, 0, -1, -1, true, 'p', 'i', true, false, false, true, 0, { 0 } }, \
- { 1249, {"attcacheoff"},  23, -1, 0,	4,	9, 0, -1, -1, true, 'p', 'i', true, false, false, true, 0, { 0 } }, \
- { 1249, {"atttypmod"},	  23, -1, 0,	4, 10, 0, -1, -1, true, 'p', 'i', true, false, false, true, 0, { 0 } }, \
- { 1249, {"attbyval"},	  16, -1, 0,	1, 11, 0, -1, -1, true, 'p', 'c', true, false, false, true, 0, { 0 } }, \
- { 1249, {"attstorage"},   18, -1, 0,	1, 12, 0, -1, -1, true, 'p', 'c', true, false, false, true, 0, { 0 } }, \
- { 1249, {"attalign"},	  18, -1, 0,	1, 13, 0, -1, -1, true, 'p', 'c', true, false, false, true, 0, { 0 } }, \
- { 1249, {"attnotnull"},   16, -1, 0,	1, 14, 0, -1, -1, true, 'p', 'c', true, false, false, true, 0, { 0 } }, \
- { 1249, {"atthasdef"},	  16, -1, 0,	1, 15, 0, -1, -1, true, 'p', 'c', true, false, false, true, 0, { 0 } }, \
- { 1249, {"attisdropped"}, 16, -1, 0,	1, 16, 0, -1, -1, true, 'p', 'c', true, false, false, true, 0, { 0 } }, \
- { 1249, {"attislocal"},   16, -1, 0,	1, 17, 0, -1, -1, true, 'p', 'c', true, false, false, true, 0, { 0 } }, \
- { 1249, {"attinhcount"},  23, -1, 0,	4, 18, 0, -1, -1, true, 'p', 'i', true, false, false, true, 0, { 0 } }, \
- { 1249, {"attacl"},		1034, -1, 0,  -1, 19, 1, -1, -1, false, 'x', 'i', false, false, false, true, 0, { 0 } }
- 
  DATA(insert ( 1249 attrelid			26 -1 0  4   1 0 -1 -1 t p i t f f t 0 _null_));
  DATA(insert ( 1249 attname			19 -1 0 NAMEDATALEN  2 0 -1 -1 f p c t f f t 0 _null_));
  DATA(insert ( 1249 atttypid			26 -1 0  4   3 0 -1 -1 t p i t f f t 0 _null_));
--- 296,301 ----
***************
*** 405,437 **** DATA(insert ( 1249 tableoid			26 0 0  4  -7 0 -1 -1 t p i t f f t 0 _null_));
   *		pg_class
   * ----------------
   */
- #define Schema_pg_class \
- { 1259, {"relname"},	   19, -1, 0, NAMEDATALEN, 1, 0, -1, -1, false, 'p', 'c', true, false, false, true, 0, { 0 } }, \
- { 1259, {"relnamespace"},  26, -1, 0,	4,	2, 0, -1, -1, true, 'p', 'i', true, false, false, true, 0, { 0 } }, \
- { 1259, {"reltype"},	   26, -1, 0,	4,	3, 0, -1, -1, true, 'p', 'i', true, false, false, true, 0, { 0 } }, \
- { 1259, {"relowner"},	   26, -1, 0,	4,	4, 0, -1, -1, true, 'p', 'i', true, false, false, true, 0, { 0 } }, \
- { 1259, {"relam"},		   26, -1, 0,	4,	5, 0, -1, -1, true, 'p', 'i', true, false, false, true, 0, { 0 } }, \
- { 1259, {"relfilenode"},   26, -1, 0,	4,	6, 0, -1, -1, true, 'p', 'i', true, false, false, true, 0, { 0 } }, \
- { 1259, {"reltablespace"}, 26, -1, 0,	4,	7, 0, -1, -1, true, 'p', 'i', true, false, false, true, 0, { 0 } }, \
- { 1259, {"relpages"},	   23, -1, 0,	4,	8, 0, -1, -1, true, 'p', 'i', true, false, false, true, 0, { 0 } }, \
- { 1259, {"reltuples"},	   700, -1, 0, 4,	9, 0, -1, -1, FLOAT4PASSBYVAL, 'p', 'i', true, false, false, true, 0, { 0 } }, \
- { 1259, {"reltoastrelid"}, 26, -1, 0,	4, 10, 0, -1, -1, true, 'p', 'i', true, false, false, true, 0, { 0 } }, \
- { 1259, {"reltoastidxid"}, 26, -1, 0,	4, 11, 0, -1, -1, true, 'p', 'i', true, false, false, true, 0, { 0 } }, \
- { 1259, {"relhasindex"},   16, -1, 0,	1, 12, 0, -1, -1, true, 'p', 'c', true, false, false, true, 0, { 0 } }, \
- { 1259, {"relisshared"},   16, -1, 0,	1, 13, 0, -1, -1, true, 'p', 'c', true, false, false, true, 0, { 0 } }, \
- { 1259, {"relistemp"},	   16, -1, 0,	1, 14, 0, -1, -1, true, 'p', 'c', true, false, false, true, 0, { 0 } }, \
- { 1259, {"relkind"},	   18, -1, 0,	1, 15, 0, -1, -1, true, 'p', 'c', true, false, false, true, 0, { 0 } }, \
- { 1259, {"relnatts"},	   21, -1, 0,	2, 16, 0, -1, -1, true, 'p', 's', true, false, false, true, 0, { 0 } }, \
- { 1259, {"relchecks"},	   21, -1, 0,	2, 17, 0, -1, -1, true, 'p', 's', true, false, false, true, 0, { 0 } }, \
- { 1259, {"relhasoids"},    16, -1, 0,	1, 18, 0, -1, -1, true, 'p', 'c', true, false, false, true, 0, { 0 } }, \
- { 1259, {"relhaspkey"},    16, -1, 0,	1, 19, 0, -1, -1, true, 'p', 'c', true, false, false, true, 0, { 0 } }, \
- { 1259, {"relhasrules"},   16, -1, 0,	1, 20, 0, -1, -1, true, 'p', 'c', true, false, false, true, 0, { 0 } }, \
- { 1259, {"relhastriggers"},16, -1, 0,	1, 21, 0, -1, -1, true, 'p', 'c', true, false, false, true, 0, { 0 } }, \
- { 1259, {"relhassubclass"},16, -1, 0,	1, 22, 0, -1, -1, true, 'p', 'c', true, false, false, true, 0, { 0 } }, \
- { 1259, {"relfrozenxid"},  28, -1, 0,	4, 23, 0, -1, -1, true, 'p', 'i', true, false, false, true, 0, { 0 } }, \
- { 1259, {"relacl"},		 1034, -1, 0, -1, 24, 1, -1, -1, false, 'x', 'i', false, false, false, true, 0, { 0 } }, \
- { 1259, {"reloptions"},  1009, -1, 0, -1, 25, 1, -1, -1, false, 'x', 'i', false, false, false, true, 0, { 0 } }
- 
  DATA(insert ( 1259 relname			19 -1 0 NAMEDATALEN	1 0 -1 -1 f p c t f f t 0 _null_));
  DATA(insert ( 1259 relnamespace		26 -1 0 4   2 0 -1 -1 t p i t f f t 0 _null_));
  DATA(insert ( 1259 reltype			26 -1 0 4   3 0 -1 -1 t p i t f f t 0 _null_));
--- 327,332 ----
***************
*** 470,476 **** DATA(insert ( 1259 tableoid			26 0 0  4  -7 0 -1 -1 t p i t f f t 0 _null_));
   *
   * pg_index is not bootstrapped in the same way as the other relations that
   * have hardwired pg_attribute entries in this file.  However, we do need
!  * a "Schema_xxx" macro for it --- see relcache.c.
   * ----------------
   */
  #define Schema_pg_index \
--- 365,373 ----
   *
   * pg_index is not bootstrapped in the same way as the other relations that
   * have hardwired pg_attribute entries in this file.  However, we do need
!  * a "Schema_xxx" macro for it --- see relcache.c.  But since there are no
!  * DATA() statements for pg_index in this file (or anywhere else), this can't
!  * be automatically generated, so we hard-code it here.
   * ----------------
   */
  #define Schema_pg_index \
*** a/src/include/catalog/pg_proc.h
--- b/src/include/catalog/pg_proc.h
***************
*** 10,18 ****
   * $PostgreSQL$
   *
   * NOTES
!  *	  The script catalog/genbki.sh reads this file and generates .bki
!  *	  information from the DATA() statements.  utils/Gen_fmgrtab.sh
!  *	  generates fmgroids.h and fmgrtab.c the same way.
   *
   *	  XXX do NOT break up DATA() statements into multiple lines!
   *		  the scripts are not as smart as you might think...
--- 10,18 ----
   * $PostgreSQL$
   *
   * NOTES
!  *	  The script catalog/gen_catalog.pl reads this file and generates .bki
!  *	  information from the DATA() statements; it also generates fmgroids.h
!  *	  and fmgrtab.c.
   *
   *	  XXX do NOT break up DATA() statements into multiple lines!
   *		  the scripts are not as smart as you might think...
